/* $Id: wmh.h,v 1.3 1999/06/08 00:38:20 marcus Exp $
******************************************************************************

   LibGGI BSE extension API header file

   Copyright (C) 1999 Andreas Beck	[becka@ggi-project.org]
  
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#ifndef _GGI_BSE_H
#define _GGI_BSE_H

#include <ggi/ggi.h>

#ifdef HAVE_STRUCT_GGIBSE
typedef struct ggibse *ggibse_t;
#else
typedef void *ggibse_t;
#endif

__BEGIN_DECLS

enum bsetype { 
	BSE_BOB,		/* Blitter object. Destructs display memory beneath. */
	BSE_SPRITE,		/* Real sprite. Does not affect display memory. */
	BSE_EMUSPRITE,		/* Emulated sprite. Saves display memory underneath. */
};

int ggiBseInit(void);
int ggiBseExit(void);

int ggiBseAttach(ggi_visual_t vis);
int ggiBseDetach(ggi_visual_t vis);

/* Allocating/Destroying a BSE.
 */
ggibse_t	ggiBseAllocate	(ggi_visual_t vis, int width, int height, enum bsetype type);
int		ggiBseFree	(ggibse_t bob);

/* Populate the sprite
 */
int ggiBseSetImage	(ggibse_t bob,ggi_visual_t vis,int x,int y,
			unsigned char *alpha);

/* Move the sprite around.
 */
int ggiBseMove		(ggibse_t bob,int x,int y);

/* Hiding and showing
 */
int ggiBseHide		(ggibse_t bob);
int ggiBseShow		(ggibse_t bob);

/* These are for EMU_SPRITEs. You can use these for hiding them for drawing.
 */
int ggiBseSoftHide	(ggibse_t bob);
int ggiBseSoftShow	(ggibse_t bob);

/* Some helpers for generating alpha arrays on the fly. They malloc the 
 * resulting array, so be sure to free() it again. The bob argument is used
 * to determine the size.
 */
unsigned char *ggiBseMakeAlphaFromPixelkey(ggibse_t bob,
	ggi_visual_t vis,int x,int y,ggi_pixel pix) ;
unsigned char *ggiBseMakeAlphaFromColorkey(ggibse_t bob,
	ggi_visual_t vis,int x,int y,ggi_color *col) ;

/* Some helpers for _CHANGING_ alpha arrays on the fly. As above, but they
 * operate on an existing alpha array. No malloc is performed.
 */

/* Arbitrary transforms :
 */
unsigned char *ggiBseChangeAlphaByFunction(ggibse_t bob, 
	unsigned char *alpha,
	unsigned char (*func)(unsigned char in,void *parms),void *parms);

/* "harden" an alpha-channel. Especially useful for speeding things up.
 * A yes/no type alphachannel can be handled much more efficiently.
 */
unsigned char *ggiBseChangeAlphaTreshold(ggibse_t bob, 
	unsigned char *alpha, unsigned char treshold);

/* Useful if you need the "negative" of an alpha channel.
 */
unsigned char *ggiBseChangeAlphaInvert(ggibse_t bob, 
	unsigned char *alpha);

__END_DECLS

#endif /* _GGI_BSE_H */
